package com.qtzl.sample.config;

import cn.com.taiji.common.manager.AbstractManager;
import cn.com.taiji.common.model.BaseModel;
import com.qtzl.sample.manager.feign.SmpClient;
import com.txffp.api.core.manager.comm.ApiCmd;
import com.txffp.api.core.manager.comm.JsonApiHandler;
import com.txffp.api.core.manager.comm.JsonCommManager;
import com.txffp.api.core.manager.comm.annotation.Api;
import com.txffp.api.core.manager.comm.annotation.ApiHandler;
import com.txffp.api.core.manager.comm.annotation.EnableJsonApiHandler;
import com.txffp.api.core.manager.comm.annotation.EnableRegisterApi;
import com.txffp.api.core.manager.tools.PackageTools;
import com.txffp.smp.manager.model.comm.protocol.api.ApiAddRequest;
import com.zgglyun.common.model.EncryptType;
import com.zgglyun.common.model.SignType;
import com.zgglyun.common.model.json.JsonRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;


@EnableRegisterApi(appId = "52030117", basePackages = {"com.qtzl.sample.manager.model.protocol"})
@EnableJsonApiHandler(basePackages = { "com.qtzl.sample.manager.handler" })
@Configuration
public class ApiConfig extends AbstractManager {

	@Autowired
	private SmpClient smpClient;

	@Value("${app.autoRegisterApi}")
	private boolean autoRegisterApi;

	/**
	 *
	 * 向SMP注册API
	 *
	 */
	@PostConstruct
	public void registerApi() {
	   EnableRegisterApi enable=ApiConfig.class.getAnnotation(EnableRegisterApi.class);
	   if( enable==null || !autoRegisterApi)return;
	   logger.info(" start register api to smp......");
       for(String packageName:enable.basePackages()) {
           PackageTools.listClasses(packageName,Api.class).forEach(c->{
           			JsonRequest req =toJsonRequest(toReigsterReqeuset(c),enable.appId());
           			//smpClient.jsonApi(req);
           		});
       }
    }


	  public JsonRequest toJsonRequest(BaseModel model,String appId) {
		   JsonRequest request=new JsonRequest();
		   LocalDateTime sendTime=LocalDateTime.now();
		  String fileName = (String) model.getPropertyValue("systemId") + "_" + model.getPropertyValue("serviceType") + "_" + model.getPropertyValue("cmd")+ "_";
		  request.setFilename(fileName + "REQ_"+appId+"_" +sendTime.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS"))+".json");
		   request.setSignType(SignType.NONE);
		   request.setEncryptType(EncryptType.NONE);
		   request.setVersion("1.0");
		   request.setTimestamp(sendTime);
		   request.setBizContent(model.toJson());
		   return request;
	 }




	@Bean
	public JsonCommManager jsonCommManager(ApplicationContext applicationContext) {
		JsonCommManager jsonCommManager=new JsonCommManager();
		EnableJsonApiHandler enbale =ApiConfig.class.getAnnotation(EnableJsonApiHandler.class);
		if(enbale==null) {
			return jsonCommManager;
		}
		for(String packageName:enbale.basePackages()) {
			PackageTools.listClasses(packageName,ApiHandler.class).forEach(c->{
				String sampleName=c.getSimpleName();
				String defaultName=sampleName.substring(0, 1).toLowerCase()+sampleName.substring(1, sampleName.length());
				ApiHandler tag=c.getAnnotation(ApiHandler.class);
				String name=tag.value().equals("")?defaultName:tag.value() ;
				JsonApiHandler jsonApiHandler= applicationContext.getBean(name, JsonApiHandler.class);
				logger.info("register json api handler {},apikey is {}",name,jsonApiHandler.getApikey());
				jsonCommManager.addHandler(jsonApiHandler.getApikey(), jsonApiHandler);
			});
		}
		return jsonCommManager;
	}



	private ApiAddRequest toReigsterReqeuset(Class<?> clazz){
		try {
			Api api=clazz.getAnnotation(Api.class);
			ApiAddRequest request=new ApiAddRequest();
			Class<?> cmdClazz=api.cmdClazz();
			ApiCmd cmd=(ApiCmd)cmdClazz.getMethod("valueOf", String.class).invoke(null, api.cmd());
			request.setSystemId(cmd.getSystemId());
			request.setSystemName(cmd.getSystemName());
			request.setServiceType(cmd.getSerivceType());
			request.setServiceTypeName(cmd.getSerivceTypeName());
			request.setCmd(cmd.getCmd());
			request.setCmdName(cmd.getCmdName());
			request.setStatus("USING");
			logger.info("register api to smp,the serviceType is {},the cmd is {}",
					request.getServiceType(),request.getCmd());
			return request;
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException("注册失败");
		}
	}
}
