package com.qtzl.sample.manager.tools;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import cn.com.taiji.common.pub.StringTools;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.web.multipart.MultipartFile;

/**
 * 
 * Copyright © 2018 Hui liantong information technology co.LTD All rights reserved.
 * @Description: 文件工具类
 * @author: Li Bangming;qq:1715592561     
 * @date: 2018年10月15日 下午1:16:47
 */
public class FileUtil{
	private static Logger logger=LogManager.getLogger(FileUtil.class);

	/**
	 * 文件后缀名
	 */
	public static String JPG_FILE_SUFFIX=".jpg";//.jpg文件格式
	public static String PNG_FILE_SUFFIX=".png";//.png文件格式
	public static String ZIP_FILE_SUFFIX=".zip";//.zip文件格式
	public static String XLS_FILE_SUFFIX=".xls";//.xls文件格式
	public static String XLSX_FILE_SUFFIX=".xlsx";//.xlsx文件格式
	
	
	/**
	 * 
	 * @Description: 获取文件临时目录
	 * @author Li Bangming;
	 * @date 2018年4月17日 下午4:13:18
	 */
	public static String getTempFilePath(){
		String templFilePath=System.getProperty("java.io.tmpdir") +"aftersaleFiles"+ File.separator;
		File tempDirectoryFile=new File(templFilePath);
		if(!tempDirectoryFile.exists()){
			tempDirectoryFile.mkdirs();
		}
		return templFilePath;
	}
	/**
	 * 
	 * @Description: 获取文件临时目录
	 * @author Li Bangming;
	 * @date 2018年4月17日 下午4:13:18
	 */
	public static String getTempFilePath(String projectName){
		String templFilePath=System.getProperty("java.io.tmpdir") +projectName+ File.separator;
		File tempDirectoryFile=new File(templFilePath);
		if(!tempDirectoryFile.exists()){
			tempDirectoryFile.mkdirs();
		}
		return templFilePath;
	}
	/**
	 * @Description 复制文件
	 * @author Li Bangming
	 * @date 2016年8月17日 下午4:13:18
	 * @param srcFile 源文件
	 * @param targetFil 目标文件
	 */
	public static boolean copyFile(File srcFile, File targetFile) {
		String parent=targetFile.getParent();
		File directory=new File(parent);
		if(!directory.exists()){
			directory.mkdirs();
		}
		BufferedInputStream bis = null;
		BufferedOutputStream bos = null;
		try {
			bis = new BufferedInputStream(new FileInputStream(srcFile));
			bos = new BufferedOutputStream(new FileOutputStream(targetFile));
			byte[] bts = new byte[1024];
			int len = -1;
			while ((len = bis.read(bts)) != -1) {
				bos.write(bts, 0, len);
			}
			return true;
		} catch (Exception e) {
			System.out.println(e.getLocalizedMessage());
			return false;
		} finally {
			if (bis != null) {
				try {
					bis.close();
				} catch (IOException e) {
					System.out.println(e.getLocalizedMessage());
				}
			}
			if (bos != null) {
				try {
					bos.close();
				} catch (IOException e) {
					System.out.println(e.getLocalizedMessage());
				}
			}
		}
	}
	/**
	 * @Description: 压缩为zip文件
	 * @author Li Bangming;
	 * @date 2018年4月26日 下午1:48:38
	 * @param zipFilePath zip文件路径
	 * @files 需要压缩的文件List
	 */
    public static File compressZip(String zipFilePath, List<File> files) {  
        ZipOutputStream zipOutputStream = null;  
        File file = new File(zipFilePath);  
        try {
        	 zipOutputStream = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(file)));  
             FileInputStream fileInputStream = null;  
             byte[] buf = new byte[1024];  
             int len = 0;  
             if(files!=null && files.size() > 0){  
                 for(File excelFile:files){  
                     String fileName = excelFile.getName();  
                     fileInputStream = new FileInputStream(excelFile);  
                     //放入压缩zip包中;  
                     zipOutputStream.putNextEntry(new ZipEntry(fileName));  
                     //读取文件;  
                     while((len=fileInputStream.read(buf)) >0){  
                         zipOutputStream.write(buf, 0, len);  
                     }  
                     //关闭;  
                     zipOutputStream.closeEntry();  
                     if(fileInputStream != null){  
                         fileInputStream.close();  
                     }  
                 }  
             }  
             if(zipOutputStream !=null){  
                 zipOutputStream.close();  
             }  
		} catch (Exception e) {
			e.printStackTrace();
		}
        return file;  
    }  
	
	/**
	 * @Description: 获取文件扩展名
	 * @author Li Bangming;
	 * @date 2018年4月26日 下午1:48:38
	 * @param file 文件
	 */
	public static String getFileExtensionName(File file){
		if(file==null){
			return null;
		}
		return getFileNameExtensionName(file.getName());
	}
	
	/**
	 * @Description: 获取文件扩展名
	 * @author Li Bangming;
	 * @date 2018年4月26日 下午1:48:38
	 * @param file 文件
	 */
	public static String getFileNameExtensionName(String fileName){
		if(fileName==null){
			return null;
		}
		if(fileName.contains(".")){
			return fileName.substring(fileName.lastIndexOf("."), fileName.length());
		}
		return null;
	}
	 /**
	 * @Description:解压Zip文件
	 * @author:Li Bangming;@date:2018年5月10日
	 * @param zipFile 压缩文件
	 * @param fileDirectory 解压目录
	  */
	  public static List<File> unZip(File srcFile,String fileDirectory) {  
		  List<File> files=new ArrayList<File>();
		  if(srcFile==null||!srcFile.exists()){
			  System.out.println("压缩文件不存在");
			  return files;
		  }
		  if(!getFileExtensionName(srcFile).equals(ZIP_FILE_SUFFIX)){
			  System.out.println("压缩文件类型不支持");
			  return files;
		  }
		  OutputStream os = null;
	        InputStream is = null;
	        ZipFile zipFile = null;
	        try {
	            zipFile = new ZipFile(srcFile.getAbsolutePath(),Charset.forName("GBK"));
	            Enumeration<?> entryEnum = zipFile.entries();
	            if (null != entryEnum) {
	                ZipEntry zipEntry = null;
	                while (entryEnum.hasMoreElements()) {
	                    zipEntry = (ZipEntry) entryEnum.nextElement();
	                    if (zipEntry.getSize() > 0) {
	                        // 文件
	                        File targetFile = FileUtil.buildFile(fileDirectory+ File.separator + zipEntry.getName());
	                        
	                        os = new BufferedOutputStream(new FileOutputStream(targetFile));
	                        is = zipFile.getInputStream(zipEntry);
	                        byte[] buffer = new byte[4096];
	                        int readLen = 0;
	                        while ((readLen = is.read(buffer, 0, 4096)) >= 0) {
	                            os.write(buffer, 0, readLen);
	                            os.flush();
	                        }
	                        is.close();
	                        os.close();
	                        files.add(targetFile);
	                    } 
	                    if (zipEntry.isDirectory()) {
	                    	String pathTemp =  fileDirectory + File.separator
	                                + zipEntry.getName();
	                    	File file = new File(pathTemp);
	                    	file.mkdirs();
	                        continue;
	                    }
	                }
	            }
	       } catch (Exception e) {
	            e.printStackTrace();
	        } finally {
	        	try {
	        		 if(null != zipFile){
	 	            	zipFile.close();
	 	                zipFile = null;
	 	            }
	 	            if (null != is) {
	 	                is.close();
	 	            }
	 	            if (null != os) {
	 	                os.close();
	 	            }
				} catch (Exception e2) {
					e2.printStackTrace();
				}
	        }
	     return files;
	}
	  
	/**
	* @Description:根据路径创建文件,如果目录不存在，则创建目录
	* @author:Li Bangming;@date:2018年5月10日
	* @param path 路径:目录/文件
	* @return
	 */
    public static File buildFile(String path) {
    	File file=new File(path);
    	if(file==null||!file.exists()){
    		boolean isDirectory=!path.contains(".");
    		if(isDirectory){
    			file.mkdirs();
    		}else{
    			File parent=file.getParentFile();
    			if(!parent.exists()){
    				parent.mkdirs();
    			}
    		}
    	}
    	return file;
	}
	/**
	* @Description:根据路径创建文件的目录,如果目录不存在，则创建目录
	* @author:Li Bangming;@date:2018年5月10日
	* @param path 路径:目录/文件
	* @return
	 */
    public static boolean buildFileDirectory(String path) {
    	File file=new File(path);
    	try {

        	if(file==null||!file.exists()){
        		boolean isDirectory=!path.contains(".");
        		if(isDirectory){
        			file.mkdirs();
        		}else{
        			File parent=file.getParentFile();
        			if(!parent.exists()){
        				parent.mkdirs();
        			}
        		}
        	}
		} catch (Exception e) {
	    	return false;
		}
    	return true;
	}
	/**
	* @Description:multipartFile转File
	* @author:Li Bangming;@date:2018年8月8日
	* @param path 路径:目录/文件
	* @return
	 */
	public static File multipartFileCovertToFile(MultipartFile multipartFile) {
		if(multipartFile==null) {
			return null;
		}
		 // 获取文件名
        String fileName = multipartFile.getOriginalFilename();
        if(StringTools.hasText(fileName)){
        	return null;
        }
        // 获取文件后缀
        String prefix=fileName.substring(fileName.lastIndexOf("."));
        String templeFileName=fileName.substring(0, fileName.lastIndexOf("."));
        File file =null;
		try {
			file = File.createTempFile(UUID.randomUUID().toString().replace("-", "")+templeFileName, prefix);
			multipartFile.transferTo(file);
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return file;

	}
	/**
	 * @Description: 将一个文本内容写入txt文件
	 * @author: Li Bangming;qq:1715592561     
	 * @date: 2019年9月18日 下午5:34:25 
	 * @param content 文本内容
	 * @param filePath 文件地址
	 * @param isAppend 是否追加内容
	 * @param isPrintLine 是否换行追加
	 * @return
	 */
	public static File witeTxtFile(String content,String filePath,boolean isAppend,boolean isPrintLine){
		File file=null;
		//
		file =buildFile(filePath);
		try {
			//如果不是追加内容，则先删除旧的
			if(!isAppend){
				if(file!=null&&file.exists()){
					file.delete();
				}
			}
			FileWriter fw = new FileWriter(filePath, true);
	        BufferedWriter bw = new BufferedWriter(fw);
			//是否换行
	        if(isPrintLine){
	        	if(file.length()>0){
		            bw.append("\n");
	        	}
			}
	        
	        //是否追加内容
	        if(isAppend){
	            bw.append(content);
	        }else{
				bw.write(content);
	        }
			
			bw.close();
	        fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return file;
	}
	
	/**
	 * 
	 * @Description: 解析文件编码格式
	 * @author Li Bangming;qq:1715592561
	 * @date 2020年11月6日
	 * @param path
	 * @return String
	 * @throws
	 */
	public static String resolveCharset(String path) {

		String code = "gb2312"; // 或GBK
		try {
			InputStream inputStream = new FileInputStream(path);
			byte[] head = new byte[3];
			inputStream.read(head);
			if (head[0] == -1 && head[1] == -2)
				code = "UTF-16";
			else if (head[0] == -2 && head[1] == -1)
				code = "Unicode";
			else if (head[0] == -17 && head[1] == -69 && head[2] == -65)
				code = "UTF-8";

			inputStream.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
		return code;
	}

	/**
	 * 
	 * @Description: 检测文件是否包含编码字符开头
	 * @author Li Bangming;qq:1715592561
	 * @date 2020年11月6日
	 * @param path
	 * @return String
	 * @throws
	 */
	public static boolean containCharset(String path) {

		boolean isContain=false;
		InputStream inputStream = null;
		try {
			inputStream = new FileInputStream(path);
			byte[] head = new byte[3];
			inputStream.read(head);
			if (head[0] == -1 && head[1] == -2){
				isContain=true;
			}
			else if (head[0] == -2 && head[1] == -1){
				isContain=true;
			}
			else if (head[0] == -17 && head[1] == -69 && head[2] == -65){
				isContain=true;
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally{
			if(inputStream!=null){
				try {
					inputStream.close();
				} catch (IOException e) {
				}
			}
		}
		return isContain;
	}
	/**
	* @Description: 删除文件
	* @author Li Bangming;qq:1715592561     
	* @date 2021年2月7日 
	* @param zipFilePath
	* @param files void   
	* @throws
	 */
    public static void deleteFile(File file) {  
    	if(file==null||!file.exists()){
			return ;
		}
		try {
			file.delete();
			logger.info(file+"已删除");
		} catch (Exception e) {
			logger.info(file+"删除文件不存在");
		}
    	
    }
	
	/**
	* @Description: 删除文件
	* @author Li Bangming;qq:1715592561     
	* @date 2021年2月7日 
	* @param zipFilePath
	* @param files void   
	* @throws
	 */
    public static void deleteFiles(File ... files) {  
    	if(files==null||files.length==0){
    		return ;
    	}
    	
    	for(File file:files){
    		if(file==null||!file.exists()){
    			continue;
    		}
    		try {
    			file.delete();
				logger.info(file+"已删除");
			} catch (Exception e) {
				logger.info(file+"删除文件不存在");
			}
    		
    	}
    	
    }
	/**
	* @Description: 检测文件名是否包含扩展名
	* @author Li Bangming;qq:1715592561     
	* @date 2021年9月28日 
	* @param fileName
	* @return boolean   
	* @throws
	 */
	public static boolean checkContainsExtensionName(String fileName){
		if(fileName==null){
			return false;
		}
		if(!fileName.contains(".")){
			return false;
		}
		
		String extensionName= fileName.substring(fileName.lastIndexOf("."), fileName.length());
		if(extensionName.length()==1){
			return false;
		}
		return true;
	}
	
	/**
	* @Description: 获取目录下面所有文件
	* @author Li Bangming;qq:1715592561     
	* @date 2022年3月8日
	* @param directoryPath
	* @return
	 */
    public static List<File> getDirFiles(String directoryPath) {
        List<File> list = new ArrayList<File>();
        File baseFile = new File(directoryPath);
        if (baseFile.isFile() || !baseFile.exists()) {
            return list;
        }
        File[] files = baseFile.listFiles();
        for (File file : files) {
            if (!file.isDirectory()) {
                list.add(file);
            }
        }
        return list;
    }
    /**
    * @Description: 根据文件路径获取文件名
    * @author Li Bangming;qq:1715592561     
    * @date 2022年3月8日
    * @param filePath
    * @return sdc.zip
     */
    public static String getPathFileName(String filePath){
    	String fileName=null;
    	String [] pathInfos=filePath.split("/");

		if(!filePath.contains(".")){
			return fileName;
		}
    	
    	if(filePath.contains("/")){
    		pathInfos=filePath.split("/");
    	}else if(filePath.contains("\\")){
    		pathInfos=filePath.split("\\\\");
    	}else{
    		return fileName;
    	}
		if(pathInfos.length==0){
			return fileName;
		}
		
		fileName=pathInfos[pathInfos.length-1];
		return fileName;
    }
    
    /**
    * @Description: 获取文件类型，例如"E:\\微信图片_20201110195856_副本.jpg"  返回  jpg
    * @author Li Bangming;qq:1715592561     
    * @date 2021年1月11日 
    * @param file
    * @return String   
    * @throws
     */
    public static String getFileType(File file){
    	if(file==null){
    		return null;
    	}
    	String originalFileName=file.getName();
    	String fileNameSuffix=originalFileName.substring(originalFileName.indexOf("."), originalFileName.length());
		String fileType=fileNameSuffix.replace(".", "");
		return fileType;
    }
}

