
package com.qtzl.sample.manager.tools;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONObject;

/**
 * http 请求黔通中台公共模板类
 * @author hou yi
 * @date 2022年7月15日 上午10:52:00
 */
@Service
public class QtHttpTemplate {
	@Value("${commProperties.ztAppId}")
	private String appId;

	@Value("${commProperties.ztToken}")
	private String token;

	@Value("${commProperties.ztZip}")
	private String zip;

	@Value("${commProperties.ztKeyType}")
	private String keyType;

	@Value("${commProperties.ztKey}")
	private String key;

	@Value("${commProperties.ztStamp}")
	private String stamp;

	@Value("${commProperties.ztSign}")
	private String sign;

	@Value("${commProperties.ztUrl}")
	private String ztUrl;

	@Value("${commProperties.ztChannelId}")
	private String channelId;

	@Value("${commProperties.ztAgentId}")
	private String agentId;

	@Value("${commProperties.ztStaffId}")
	private String staffId;

	@Value("${commProperties.ztTerminalId}")
	private String terminalId;

	@Value("${commProperties.ztChannelType}")
	private String channelType;
	
	/***
	 * 执行http请求
	 * @param qtCodeEnum code参数枚举类
	 * @param paramMap 接口必填参数map集合
	 * @throws Exception 、
	 * @return 接口返回数据 json 串
	 */
	public String execute(QtCodeEnum qtCodeEnum, Map<String, String> paramMap) throws Exception {
		// 
		Map<String, String> commonParam = verifyAndInit(qtCodeEnum);
		String resInfo = null;
		HttpResponse httpResponse;
		try {
			httpResponse = execute(paramMap, commonParam);
			resInfo = EntityUtils.toString(httpResponse.getEntity(), "UTF-8");
		} catch (IOException e) {
			e.printStackTrace();
			resInfo = e.getMessage();
			throw new Exception("http请求中台转发失败,请检查网络或配置文件");
		} catch (Exception e) {
			e.printStackTrace();
			resInfo = e.getMessage();
			throw new Exception("http请求中台转发失败,请联系管理员检查网络或配置文件");
		}
		// 中台接口 rcode="0"时表示成功 否则失败
		String rcode = FastJsonTools.getPropertyValueFromJsonData("rcode", resInfo);
		if (!"0".equals(rcode) && !"1030002".equals(rcode)) {
			String rmsg = FastJsonTools.getPropertyValueFromJsonData("rmsg", resInfo);
			throw new Exception("http请求中台失败:" + rmsg);
		}
		String data = FastJsonTools.getPropertyValueFromJsonData("data", resInfo);
		if (StringUtils.isEmpty(data)) {
			throw new Exception("http请求中台返回结果为空");
		}
		return data;
	}
	
	private Map<String, String> verifyAndInit(QtCodeEnum qtCodeEnum) throws Exception {
		// 参数校验并设置对应参数
		if (null == qtCodeEnum) {
			throw new Exception("http请求失败，枚举参数不能为空");
		}
		Map<String, String> commonParam = new HashMap<String, String>();
		commonParam.put("appId", appId);
		commonParam.put("code", qtCodeEnum.getCode());
		commonParam.put("token", token);
		commonParam.put("zip", zip);
		commonParam.put("keyType", keyType);
		commonParam.put("key", key);
		commonParam.put("stamp", stamp);
		commonParam.put("sign", sign);
		commonParam.put("channelId", channelId);
		commonParam.put("agentId", agentId);
		commonParam.put("staffId", staffId);
		commonParam.put("terminalId", terminalId);
		commonParam.put("channelType", channelType);
		return commonParam;
	}
	
	/**
	 * 执行请求中台
	 * 
	 * @param interfaceParam 业务接口参数
	 * @param commonParam    中台公共参数
	 * @return
	 * @throws Exception
	 */
	private HttpResponse execute(Map<String, String> interfaceParam, Map<String, String> commonParam)throws Exception{
		HttpPost configHttpClientPost = configHttpClientPost(interfaceParam, commonParam, ztUrl);
		HttpClient httpClient = HttpClients.createDefault();
		HttpResponse execute = null;
		try {
			execute = httpClient.execute(configHttpClientPost);
		} catch (ClientProtocolException e) {
			e.printStackTrace();
			throw new Exception(e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
		return execute;
	}

	private HttpPost configHttpClientPost(Map<String, String> interfaceParam, Map<String, String> commonParam,
			String url) throws Exception {
		HttpPost httpPost = new HttpPost(url);
		if (commonParam == null || commonParam.size() == 0)
			throw new Exception("未传入接口公共参数");
		List<NameValuePair> params = new ArrayList<NameValuePair>();
		for (String param : commonParam.keySet()) {
			String value = commonParam.get(param);
			params.add(new BasicNameValuePair(param, value));
		}
		if (interfaceParam != null && interfaceParam.size() > 0) {			
			params.add(new BasicNameValuePair("data", JSONObject.toJSONString(interfaceParam)));
		}
		UrlEncodedFormEntity reqEntity;
		try {
			reqEntity = new UrlEncodedFormEntity(params, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			throw new Exception("组装请求出错");
		}
		httpPost.setEntity(reqEntity);
		// 设置超时为2分钟
		int timeout = 120;
		RequestConfig defaultRequestConfig = RequestConfig.custom().setSocketTimeout(timeout * 1000)
				.setConnectTimeout(timeout * 1000).setConnectionRequestTimeout(timeout * 1000).build();
		httpPost.setConfig(defaultRequestConfig);
		return httpPost;
	}
	

}
