package com.qtzl.sample.manager.tools;

import cn.com.taiji.common.manager.AbstractManager;
import com.jcraft.jsch.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.Properties;

public class SftpTools {

    private static Logger logger = LoggerFactory.getLogger(SftpTools.class);

    private static Session sshSession;

    /**

     * 连接sftp服务器

     * @param host ftp地址

     * @param port 端口

     * @param userName 账号

     * @param password 密码

     * @return

     */

    public static ChannelSftp sftpConnection(String host,int port, String userName, String password) throws JSchException {

        JSch jsch = new JSch();

        ChannelSftp channelSftp;

        try {

            jsch.getSession(userName, host, port);

            sshSession = jsch.getSession(userName, host, port);

            sshSession.setPassword(password);

            Properties properties = new Properties();

            properties.put("StrictHostKeyChecking", "no");

            sshSession.setConfig(properties);

            sshSession.connect();

            Channel channel = sshSession.openChannel("sftp");

            channel.connect();

            channelSftp = (ChannelSftp) channel;

        }catch (JSchException e){

            e.printStackTrace();
            throw e;
        }

        return channelSftp;

    }

    /**

     *@description 退出Sftp服务器登录

     *@return

     **/

    public static void sftpClose(ChannelSftp channelSftp){

        if (channelSftp != null) {

            if (channelSftp.isConnected()){

                channelSftp.disconnect();

            }

        }

    }

    /**

     * 关闭session

     */

    public static void sessionClose(){

        if (sshSession != null) {

            if (sshSession.isConnected()){

                sshSession.disconnect();

                sshSession = null;

            }

        }

    }

    /**

     * 下载sftp文件

     * @param sftp

     * @param newFileName 新文件名称

     * @param path 文件路径

     * @param fileName 文件名称

     * @param downUrl 下载到本地的路径

     * @throws Exception

     */

    public static void downSftpFile(ChannelSftp sftp, String newFileName,String path, String fileName, String downUrl) throws Exception {

        OutputStream os=null;

        try {

            File localFile = new File(downUrl + "/" + newFileName);

            if (!localFile.getParentFile().exists()) {

                localFile.getParentFile().mkdirs();

                localFile.createNewFile();

            }

            if (path != null && !"".equals(path)) {

                sftp.cd(path);//进入所在路径

            }

            os = new FileOutputStream(localFile);

            sftp.get(path + fileName, os);

            os.close();

        }catch (Exception e){

            e.printStackTrace();

        }

    }

    public static void uploadFile(String uploadUrl, ChannelSftp sftp,
                                  String userName, String password, String host, Integer port,
                                  String ftpPath, String localPath, String fileName) {
        try {
            //1.获取会话
            JSch jSch = new JSch();
            Session session = jSch.getSession(userName, host, port);
            session.setPassword(password);
            Properties properties = new Properties();
            properties.put("StrictHostKeyChecking", "no");
            session.setConfig(properties);
            session.connect();
            //2.获取上传的管道
            Channel channel = session.openChannel("sftp");
            channel.connect();
            sftp = (ChannelSftp) channel;
            sftp.cd(ftpPath);

            //读取源文件流（客户端文件）
            InputStream client_fileInput = new FileInputStream(localPath );

            sftp.put(client_fileInput, fileName);
            //传送到服务端
            //5.关闭
            sftp.disconnect();
            client_fileInput.close();
            channel.disconnect();
            session.disconnect();
            logger.info("sftp传输成功");
        } catch (Exception e) {
            logger.info("sftp传输失败: " + e.toString());
            e.printStackTrace();
        }
    }
}
