package com.qtzl.sample.manager.tools;

import cn.com.taiji.common.pub.StringTools;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**   
 * Copyright  2020 Hui liantong information technology co.LTD All rights reserved.
 * 
 * @Description:  txt工具类
 * @author: Li Bangming;qq:1715592561     
 * @date: 2020年4月16日
 */
public class TxtUtil {

    /**
    * @Description: 读取txt文件
    * @author Li Bangming;qq:1715592561     
    * @date 2020年4月20日 
    * @param fileName
    * @return List<Object>   
    * @throws
     */
    public static List<Object> readTxtData(String fileName,String trimStr) {
        List<Object> records = new ArrayList<Object>();
        if(!StringTools.hasText(fileName)){
            return records;
        }
        try {
            String record;
            // 设定字符集，使用带缓冲区的字符输入流BufferedReader读取文件内容
            String charset= FileUtil.resolveCharset(fileName);
            boolean startCharset=FileUtil.containCharset(fileName);
            
            BufferedReader file = new BufferedReader(new InputStreamReader(new FileInputStream(fileName), charset));
            // file.readLine(); //跳过表头所在的行
            System.out.println(file.readLine());
            // 遍历数据行并存储在名为records的ArrayList中，每一行records中存储的对象为一个String数组
            int i=0;
            while ((record = file.readLine()) != null) {
                String fields[] = record.split("\t");
                //去掉第一个字符的编码
                if(i==0&&startCharset&&fields.length>1){
                	fields[0]=fields[0].substring(1);
                }
                
                List<Object> values=new ArrayList<Object>();
              
                for(String field:fields){
                	if(!StringTools.hasText(field)&&!StringTools.hasText(trimStr)){
                		field=StringUtil.setCharset(field, "GB2312");
                		field=field.replace(trimStr, "");
                	}
                	
                	values.add(field);
                }
                records.add(values);
                i++;
            }
            // 关闭文件
            file.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
        return records;
    }
    
    /**
    * @Description: 读取txt文件
    * @author Li Bangming;qq:1715592561     
    * @date 2021年4月22日 
    * @param fileName 文件名
    * @param trimStr 去除符号
    * @param startRowIndex 读取起始行下标
    * @param readRowNum 读取行数
    * @return List<Object>   
    * @throws
     */
    public static List<Object> readTxtData(String fileName,String trimStr,long startRowIndex, long readRowNum) {
        List<Object> records = new ArrayList<Object>();
        if(StringUtil.isBlankOrEmpty(fileName)||startRowIndex<0||readRowNum<=0){
            return records;
        }
        try {
            String record;
            // 设定字符集，使用带缓冲区的字符输入流BufferedReader读取文件内容
            String charset= FileUtil.resolveCharset(fileName);
            boolean startCharset=FileUtil.containCharset(fileName);
            
            BufferedReader file = new BufferedReader(new InputStreamReader(new FileInputStream(fileName), charset));
            // file.readLine(); //跳过表头所在的行

            // 遍历数据行并存储在名为records的ArrayList中，每一行records中存储的对象为一个String数组
            long i=0;
            while ((record = file.readLine()) != null) {
            	
            	if(i>=startRowIndex&&i<startRowIndex+readRowNum){
                    String fields[] = record.split("\t");
                    //去掉第一个字符的编码
                    if(i==0&&startCharset&&fields.length>1){
                    	fields[0]=fields[0].substring(1);
                    }
                    
                    List<Object> values=new ArrayList<Object>();
                  
                    for(String field:fields){
                    	if(!StringUtil.isBlankOrEmpty(field)&&!StringUtil.isBlankOrEmpty(trimStr)){
                    		field=StringUtil.setCharset(field, charset);
                    		field=field.replace(trimStr, "");
                    	}
                    	
                    	values.add(field);
                    }
                    records.add(values);
            	}
                
                i++;
            }
            // 关闭文件
            file.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
        return records;
    }
    
    /**
     * @Description: 读取txt文件
     * @author Li Bangming;qq:1715592561     
     * @date 2020年4月20日 
     * @param fileName
     * @return List<Object>   
     * @throws
      */
     public static List<Object> readTxtData(String fileName) {
    	 return readTxtData(fileName, null);
     }
     /**
     * @Description: 获取文件总行数
     * @author Li Bangming;qq:1715592561     
     * @date 2021年4月22日 
     * @param fileName
     * @return long   
     * @throws
      */
     public static long getTotalRowNum(String fileName){
		File file = new File(fileName);
		if (file.exists()) {
			try {
				FileReader fileReader = new FileReader(file);
				LineNumberReader lineNumberReader = new LineNumberReader(
						fileReader);
				lineNumberReader.skip(Long.MAX_VALUE);
				long lines = lineNumberReader.getLineNumber() + 1;
				fileReader.close();
				lineNumberReader.close();
				return lines;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return 0;
    	 
     }
     
     
     
     public static void main(String[] args) {
//    	 List<Object> datas=TxtUtil.readTxtData("D:\\test\\902_20190122_1548090005235_ZTTX_Entrust_Detail.txt");
//    	 for(Object data:datas){
//    		 System.out.println(data);
//    	 }
    	 
    	 //指定行读取
    	 List<Object> datas=TxtUtil.readTxtData("D:\\test\\902_20190122_1548090005235_ZTTX_Entrust_Detail.txt",",",0,3);
    	 for(Object data:datas){
    		 System.out.println(data);
    	 }
    	 
    	 
    	 System.out.println(getTotalRowNum("D:\\test\\902_20190122_1548090005235_ZTTX_Entrust_Detail.txt"));
	 }

}
